@@ -38,7 +38,9 @@ namespace nemiver {
 struct BPColumns : public Gtk::TreeModelColumnRecord {
     Gtk::TreeModelColumn<int> id ;
     Gtk::TreeModelColumn<bool> enabled ;
+    Gtk::TreeModelColumn<Glib::ustring> address ;
     Gtk::TreeModelColumn<Glib::ustring> filename ;
+    Gtk::TreeModelColumn<Glib::ustring> function ;
     Gtk::TreeModelColumn<int> line ;
     Gtk::TreeModelColumn<IDebugger::BreakPoint> breakpoint ;
 
@@ -46,7 +48,9 @@ struct BPColumns : public Gtk::TreeModelColumnRecord {
     {
         add (id) ;
         add (enabled) ;
+        add (address) ;
         add (filename) ;
+        add (function) ;
         add (line) ;
         add (breakpoint) ;
     }
@@ -94,12 +98,17 @@ public:
         tree_view->get_selection ()->signal_changed ().connect (
                 sigc::mem_fun (*this, &Priv::on_treeview_selection_changed));
 
+        tree_view->signal_key_press_event ().connect (sigc::mem_fun (*this,
+                    &Priv::on_key_press_event));
+
         // update breakpoint list when debugger indicates that the list of
         // breakpoints has changed.
         debugger->breakpoint_deleted_signal ().connect (sigc::mem_fun
                 (*this, &Priv::on_debugger_breakpoint_deleted_signal)) ;
         debugger->breakpoints_set_signal ().connect (sigc::mem_fun
                 (*this, &Priv::on_debugger_breakpoints_set_signal)) ;
+        breakpoints_menu = load_menu ("breakpointspopup.xml",
+                "/BreakpointsPopup");
     }
 
     void build_tree_view ()
@@ -115,6 +124,8 @@ public:
         tree_view->append_column (_("ID"), get_bp_columns ().id) ;
         tree_view->append_column (_("Filename"), get_bp_columns ().filename) ;
         tree_view->append_column (_("Line"), get_bp_columns ().line) ;
+        tree_view->append_column (_("Function"), get_bp_columns ().function) ;
+        tree_view->append_column (_("Address"), get_bp_columns ().address) ;
         Gtk::CellRendererToggle *enabled_toggle =
             dynamic_cast<Gtk::CellRendererToggle*>
                                     (tree_view->get_column_cell_renderer(0));
@@ -159,8 +170,10 @@ public:
                 Gtk::TreeModel::iterator tree_iter =
                     find_breakpoint_in_model(breakmap_iter->second);
                 if (tree_iter) {
+                    LOG_DD ("Updating breakpoint " << breakmap_iter->second.number ());
                     update_breakpoint(tree_iter, breakmap_iter->second);
                 } else {
+                    LOG_DD ("Adding breakpoint " << breakmap_iter->second.number ());
                     append_breakpoint(breakmap_iter->second);
                 }
             }
@@ -213,25 +226,20 @@ public:
     void update_breakpoint(Gtk::TreeModel::iterator& a_iter,
             const IDebugger::BreakPoint &a_breakpoint)
     {
-        // we should only be 'updating' a breakpoint in the list if they have
-        // the same ID
-        THROW_IF_FAIL ((*a_iter)[get_bp_columns ().id]
-                       == a_breakpoint.number ());
         (*a_iter)[get_bp_columns ().breakpoint] = a_breakpoint;
         (*a_iter)[get_bp_columns ().enabled] = a_breakpoint.enabled () ;
-        (*a_iter)[get_bp_columns ().filename] = a_breakpoint.file_name () ;
-        (*a_iter)[get_bp_columns ().line] = a_breakpoint.line () ;
+        (*a_iter)[get_bp_columns ().id] = a_breakpoint.number () ;
+        (*a_iter)[get_bp_columns ().function] = a_breakpoint.function ();
+        (*a_iter)[get_bp_columns ().address] = a_breakpoint.address () ;
+        (*a_iter)[get_bp_columns ().filename] = a_breakpoint.file_name ();
+        (*a_iter)[get_bp_columns ().line] = a_breakpoint.line ();
     }
 
     Gtk::TreeModel::iterator append_breakpoint
                                     (const IDebugger::BreakPoint &a_breakpoint)
     {
         Gtk::TreeModel::iterator tree_iter = list_store->append();
-        (*tree_iter)[get_bp_columns ().id] = a_breakpoint.number ();
-        (*tree_iter)[get_bp_columns ().breakpoint] = a_breakpoint;
-        (*tree_iter)[get_bp_columns ().enabled] = a_breakpoint.enabled ();
-        (*tree_iter)[get_bp_columns ().filename] = a_breakpoint.file_name ();
-        (*tree_iter)[get_bp_columns ().line] = a_breakpoint.line ();
+        update_breakpoint (tree_iter, a_breakpoint);
 
         return tree_iter;
     }
@@ -286,11 +294,7 @@ public:
 
     Gtk::Widget* get_breakpoints_menu ()
     {
-        if (!breakpoints_menu) {
-            breakpoints_menu = load_menu ("breakpointspopup.xml",
-                                          "/BreakpointsPopup");
-            THROW_IF_FAIL (breakpoints_menu);
-        }
+        THROW_IF_FAIL (breakpoints_menu);
         return breakpoints_menu;
     }
 
@@ -351,28 +355,19 @@ public:
     {
         NEMIVER_TRY
         THROW_IF_FAIL(tree_view)
-        if (tree_view->get_selection ()->count_selected_rows () > 1) {
-            Glib::RefPtr<Gtk::Action> action =
-                workbench.get_ui_manager ()->get_action
-                ("/BreakpointsPopup/GoToSourceBreakpointMenuItem");
-            if (action) {
+
+        Glib::RefPtr<Gtk::Action> action = workbench.get_ui_manager ()->get_action
+            ("/BreakpointsPopup/GoToSourceBreakpointMenuItem");
+        if (action) {
+            if (tree_view->get_selection ()->count_selected_rows () > 1) {
                 action->set_sensitive (false) ;
             } else {
-                LOG_ERROR
-                    ("Could not get action "
-                     "/BreakpointsPopup/GoToSourceBreakpointMenuItem") ;
+                action->set_sensitive (true) ;
             }
         } else {
-            Glib::RefPtr<Gtk::Action> action =
-                workbench.get_ui_manager ()->get_action
-                ("/BreakpointsPopup/GoToSourceBreakpointMenuItem");
-            if (action) {
-                action->set_sensitive (true) ;
-            } else {
-                LOG_ERROR
-                    ("Could not get action "
-                     "/BreakpointsPopup/GoToSourceBreakpointMenuItem") ;
-            }
+            LOG_ERROR
+                ("Could not get action "
+                 "/BreakpointsPopup/GoToSourceBreakpointMenuItem") ;
         }
         NEMIVER_CATCH
     }
@@ -454,6 +449,15 @@ public:
         debugger->list_breakpoints ();
     }
 
+    bool on_key_press_event (GdkEventKey* event)
+    {
+        if (event && event->keyval == GDK_Delete)
+        {
+            on_breakpoint_delete_action ();
+        }
+        return false;
+    }
+
 };//end class BreakpointsView::Priv
 
 BreakpointsView::BreakpointsView (IWorkbench& a_workbench,
